@@ -1,23 +1,17 @@
 """Module contains the class to construct fuzzyfinder prompt."""
 import asyncio
 import math
-from typing import Any, Callable, Dict, List, Tuple, Union
+from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
 from prompt_toolkit.application.application import Application
 from prompt_toolkit.buffer import Buffer
-from prompt_toolkit.filters.base import FilterOrBool
 from prompt_toolkit.filters.cli import IsDone
-from prompt_toolkit.layout.containers import (
-    ConditionalContainer,
-    Float,
-    FloatContainer,
-    HSplit,
-    Window,
-)
+from prompt_toolkit.layout.containers import ConditionalContainer, HSplit, Window
 from prompt_toolkit.layout.controls import BufferControl, FormattedTextControl
 from prompt_toolkit.layout.dimension import Dimension, LayoutDimension
 from prompt_toolkit.layout.layout import Layout
 from prompt_toolkit.layout.processors import AfterInput, BeforeInput
+from prompt_toolkit.lexers.base import SimpleLexer
 from prompt_toolkit.validation import ValidationError, Validator
 from prompt_toolkit.widgets.base import Frame
 
@@ -26,6 +20,7 @@ from InquirerPy.enum import INQUIRERPY_POINTER_SEQUENCE
 from InquirerPy.exceptions import InvalidArgument
 from InquirerPy.prompts.fuzzy.fzy import fuzzy_match_py_async
 from InquirerPy.separator import Separator
+from InquirerPy.utils import InquirerPyStyle, SessionResult, calculate_height
 
 
 class InquirerPyFuzzyControl(InquirerPyUIControl):
@@ -35,7 +30,7 @@ class InquirerPyFuzzyControl(InquirerPyUIControl):
     The actual input buffer will be handled by a separate BufferControl.
 
     :param choices: list of choices to display
-    :type choices: Union[Callable[[], List[Any]], List[Any]],
+    :type choices: Union[Callable[[SessionResult], List[Any]], List[Any]],
     :param pointer: pointer symbol
     :type pointer: str
     :param marker: marker symbol for the selected choice in the case of multiselect
@@ -48,18 +43,19 @@ class InquirerPyFuzzyControl(InquirerPyUIControl):
 
     def __init__(
         self,
-        choices: Union[Callable[[], List[Any]], List[Any]],
+        choices: Union[Callable[[SessionResult], List[Any]], List[Any]],
         pointer: str,
         marker: str,
         current_text: Callable[[], str],
         max_lines: int,
+        session_result: Optional[SessionResult],
     ) -> None:
         """Construct UIControl and initialise choices."""
         self._pointer = pointer
         self._marker = marker
         self._current_text = current_text
-        self._max_lines = max_lines
-        super().__init__(choices, None)
+        self._max_lines = max_lines if max_lines > 0 else 1
+        super().__init__(choices=choices, default=None, session_result=session_result)
 
     def _format_choices(self) -> None:
         for index, choice in enumerate(self.choices):
@@ -86,7 +82,7 @@ class InquirerPyFuzzyControl(InquirerPyUIControl):
         display_choices.append(("class:pointer", self._pointer))
         display_choices.append(
             (
-                "class:fuzzy_marker",
+                "class:marker",
                 self._marker if self.choices[choice["index"]]["enabled"] else " ",
             )
         )
@@ -117,7 +113,7 @@ class InquirerPyFuzzyControl(InquirerPyUIControl):
         display_choices.append(("class:pointer", len(self._pointer) * " "))
         display_choices.append(
             (
-                "class:fuzzy_marker",
+                "class:marker",
                 self._marker if self.choices[choice["index"]]["enabled"] else " ",
             )
         )
@@ -212,21 +208,21 @@ class FuzzyPrompt(BaseComplexPrompt):
     python provider.
 
     :param message: message to display to the user
-    :type message: str
+    :type message: Union[str, Callable[[SessionResult], str]]
     :param choices: list of choices available to select
-    :type choices: Union[Callable[[], List[Any]], List[Any]],
+    :type choices: Union[Callable[[SessionResult], List[Any]], List[Any]],
     :param default: default value to insert into buffer
-    :type default: str
+    :type default: Union[str, Callable[[SessionResult], str]]
     :param pointer: pointer symbol
     :type pointer: str
     :param style: style dict to apply
-    :type style: Dict[str, str]
-    :param editing_mode: keybinding mode
-    :type editing_mode: str
+    :type style: InquirerPyStyle
+    :param vi_mode: use vi kb for the prompt
+    :type vi_mode: bool
     :param qmark: question mark symbol
     :type qmark: str
     :param transformer: transform the result to output, this is only visual effect
-    :type transformer: Callable[[str], Any]
+    :type transformer: Callable[[Any], Any]
     :param filter: a callable to filter the result, updating the user input before returning the result
     :type filter: Callable[[Any], Any]
     :param instruction: instruction to display after the message
@@ -246,23 +242,23 @@ class FuzzyPrompt(BaseComplexPrompt):
     :param max_height: max height choice window should reach
     :type max_height: Union[str, int]
     :param validate: a callable or Validator instance to validate user selection
-    :type validate: Union[Callable[[str], bool], Validator]
+    :type validate: Union[Callable[[Any], bool], Validator]
     :param invalid_message: message to display when input is invalid
     :type invalid_message: str
     :param keybindings: custom keybindings to apply
-    :type keybindings: Dict[str, List[Dict[str, Union[str, FilterOrBool]]]]
+    :type keybindings: Dict[str, List[Dict[str, Any]]]
     """
 
     def __init__(
         self,
-        message: str,
-        choices: Union[Callable[[], List[Any]], List[Any]],
-        default: str = "",
+        message: Union[str, Callable[[SessionResult], str]],
+        choices: Union[Callable[[SessionResult], List[Any]], List[Any]],
+        default: Union[str, Callable[[SessionResult], str]] = "",
         pointer: str = INQUIRERPY_POINTER_SEQUENCE,
-        style: Dict[str, str] = None,
-        editing_mode: str = "default",
+        style: InquirerPyStyle = None,
+        vi_mode: bool = False,
         qmark: str = "?",
-        transformer: Callable[[str], Any] = None,
+        transformer: Callable[[Any], Any] = None,
         filter: Callable[[Any], Any] = None,
         instruction: str = "",
         multiselect: bool = False,
@@ -272,9 +268,10 @@ class FuzzyPrompt(BaseComplexPrompt):
         info: bool = True,
         height: Union[str, int] = None,
         max_height: Union[str, int] = None,
-        validate: Union[Callable[[str], bool], Validator] = None,
+        validate: Union[Callable[[Any], bool], Validator] = None,
         invalid_message: str = "Invalid input",
-        keybindings: Dict[str, List[Dict[str, Union[str, FilterOrBool]]]] = None,
+        keybindings: Dict[str, List[Dict[str, Any]]] = None,
+        session_result: SessionResult = None,
     ) -> None:
         """Initialise the layout and create Application.
 
@@ -299,7 +296,6 @@ class FuzzyPrompt(BaseComplexPrompt):
         self._info = info
         self._task = None
         self._rendered = False
-        self._default = str(default)
         self._content_control: InquirerPyFuzzyControl
 
         keybindings = {
@@ -310,17 +306,20 @@ class FuzzyPrompt(BaseComplexPrompt):
         super().__init__(
             message=message,
             style=style,
-            editing_mode=editing_mode,
+            vi_mode=vi_mode,
             qmark=qmark,
             transformer=transformer,
             filter=filter,
             validate=validate,
             invalid_message=invalid_message,
-            height=height,
-            max_height=max_height,
             multiselect=multiselect,
             instruction=instruction,
             keybindings=keybindings,
+            session_result=session_result,
+        )
+        self._default = default if not isinstance(default, Callable) else default(self._result)  # type: ignore
+        self._dimmension_height, self._dimmension_max_height = calculate_height(
+            height, max_height, offset=3 if not self._border else 5
         )
 
         self._content_control = InquirerPyFuzzyControl(
@@ -331,6 +330,7 @@ class FuzzyPrompt(BaseComplexPrompt):
             max_lines=self._dimmension_max_height
             if not self._border
             else self._dimmension_max_height - 2,
+            session_result=session_result,
         )
 
         self._buffer = Buffer(on_text_changed=self._on_text_changed)
@@ -346,17 +346,24 @@ class FuzzyPrompt(BaseComplexPrompt):
                     AfterInput(self._generate_after_input),
                     BeforeInput(self._generate_before_input),
                 ],
+                lexer=SimpleLexer("class:input"),
             ),
         )
 
-        choice_height_dimmension = Dimension(
-            max=self._dimmension_max_height, preferred=self._dimmension_height
+        choice_height_dimmension = lambda: Dimension(
+            max=self._dimmension_max_height
+            if not self._border
+            else self._dimmension_max_height - 2,
+            preferred=self._dimmension_height,
+            min=self.content_control._height if self.content_control._height > 0 else 1,
         )
-        choice_window = Window(
-            content=self.content_control, height=choice_height_dimmension
+        self.choice_window = Window(
+            content=self.content_control,
+            height=choice_height_dimmension,
+            dont_extend_height=True,
         )
 
-        main_content_window = HSplit([input_window, choice_window])
+        main_content_window = HSplit([input_window, self.choice_window])
         if self._border:
             main_content_window = Frame(main_content_window)
         self._layout = Layout(
@@ -364,30 +371,25 @@ class FuzzyPrompt(BaseComplexPrompt):
                 [
                     message_window,
                     ConditionalContainer(
-                        FloatContainer(
-                            content=main_content_window,
-                            floats=[
-                                Float(
-                                    ConditionalContainer(
-                                        Window(
-                                            FormattedTextControl(
-                                                [
-                                                    (
-                                                        "class:validation-toolbar",
-                                                        self._invalid_message,
-                                                    )
-                                                ]
-                                            ),
-                                            dont_extend_height=True,
-                                        ),
-                                        filter=self._is_invalid,
-                                    ),
-                                    bottom=1 if self._border else 0,
-                                    left=1 if self._border else 0,
-                                )
-                            ],
+                        main_content_window, filter=~IsDone() & ~self._is_loading
+                    ),
+                    ConditionalContainer(
+                        Window(FormattedTextControl([("", "")])),
+                        filter=~IsDone(),  # force validation bar to stay bottom
+                    ),
+                    ConditionalContainer(
+                        Window(
+                            FormattedTextControl(
+                                [
+                                    (
+                                        "class:validation-toolbar",
+                                        self._invalid_message,
+                                    )
+                                ]
+                            ),
+                            dont_extend_height=True,
                         ),
-                        filter=~IsDone() & ~self._is_loading,
+                        filter=self._is_invalid & ~IsDone(),
                     ),
                 ]
             )
@@ -414,8 +416,9 @@ class FuzzyPrompt(BaseComplexPrompt):
         if not self._rendered:
             super()._after_render(application)
             if self._default:
-                self._buffer.text = self._default
-                self._buffer.cursor_position = len(self._default)
+                default_text = str(self._default)  # fix type annotation
+                self._buffer.text = default_text
+                self._buffer.cursor_position = len(default_text)
 
     def _toggle_all(self, value: bool = None) -> None:
         """Toggle all choice `enabled` status.
@@ -569,10 +572,6 @@ class FuzzyPrompt(BaseComplexPrompt):
         try:
             fake_document = FakeDocument(self.result_value)
             self._validator.validate(fake_document)  # type: ignore
-        except ValidationError:
-            self._invalid = True
-            return
-        try:
             if self._multiselect:
                 self.status["answered"] = True
                 if not self.selected_choices:
@@ -585,6 +584,8 @@ class FuzzyPrompt(BaseComplexPrompt):
                 self.status["answered"] = True
                 self.status["result"] = self.content_control.selection["name"]
                 event.app.exit(result=self.content_control.selection["value"])
+        except ValidationError:
+            self._invalid = True
         except IndexError:
             self.status["answered"] = True
             self.status["result"] = None if not self._multiselect else []
