@@ -1,4 +1,5 @@
 """Module contains the class to create an input prompt."""
+import os
 from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
 from prompt_toolkit.completion import NestedCompleter
@@ -6,12 +7,13 @@ from prompt_toolkit.completion.base import Completer
 from prompt_toolkit.filters.base import Condition
 from prompt_toolkit.keys import Keys
 from prompt_toolkit.lexers import SimpleLexer
-from prompt_toolkit.shortcuts.prompt import PromptSession
+from prompt_toolkit.shortcuts.prompt import CompleteStyle, PromptSession
 from prompt_toolkit.validation import ValidationError, Validator
 
 from InquirerPy.base import BaseSimplePrompt
-from InquirerPy.enum import INQUIRERPY_POINTER_SEQUENCE
+from InquirerPy.enum import INQUIRERPY_KEYBOARD_INTERRUPT, INQUIRERPY_POINTER_SEQUENCE
 from InquirerPy.exceptions import InvalidArgument
+from InquirerPy.utils import InquirerPyStyle, SessionResult
 
 
 class InputPrompt(BaseSimplePrompt):
@@ -20,17 +22,19 @@ class InputPrompt(BaseSimplePrompt):
     This class is used for input prompt.
 
     :param message: the question to ask
-    :type message: str
+    :type message: Union[str, Callable[[SessionResult], str]]
     :param style: a dictionary of style to apply
-    :type style: Dict[str, str]
-    :param editing_mode: the mode of editing
-    :type editing_mode: str
+    :type style: InquirerPyStyle
+    :param vi_mode: use vi kb for the prompt
+    :type vi_mode: bool
     :param default: the default result
-    :type default: str
+    :type default: Union[str, Callable[[SessionResult], str]]
     :param qmark: question qmark to display
     :type qmark: str
     :param completer: add auto completer to user input
     :type completer: Union[Dict[str, str], Completer]
+    :param multicolumn_complete: complete in multi column
+    :type multicolumn_complete: bool
     :param multiline: enable multiline mode
     :type multiline: bool
     :param validate: a callable or a validation class to validate user input
@@ -40,36 +44,39 @@ class InputPrompt(BaseSimplePrompt):
     :param transformer: a callable to transform the result, this is visual effect only
     :type transformer: Callable[[str], Any]
     :param filter: a callable to filter the result, updating the user input before returning the result
-    :type filter: Callable[[Any], Any]
+    :type filter: Callable[[str], Any]
     """
 
     def __init__(
         self,
-        message: str,
-        style: Dict[str, str] = None,
-        editing_mode: str = "default",
-        default: str = "",
+        message: Union[str, Callable[[SessionResult], str]],
+        style: InquirerPyStyle = None,
+        vi_mode: bool = False,
+        default: Union[str, Callable[[SessionResult], str]] = "",
         qmark: str = "?",
         completer: Union[Dict[str, Optional[str]], Completer] = None,
+        multicolumn_complete: bool = False,
         multiline: bool = False,
         validate: Union[Callable[[str], bool], Validator] = None,
         invalid_message: str = "Invalid input",
         transformer: Callable[[str], Any] = None,
-        filter: Callable[[Any], Any] = None,
+        filter: Callable[[str], Any] = None,
+        session_result: SessionResult = None,
         **kwargs,
     ) -> None:
         """Construct a PromptSession based on parameters and apply key_bindings."""
         super().__init__(
             message,
             style,
-            editing_mode=editing_mode,
+            vi_mode=vi_mode,
             qmark=qmark,
             validate=validate,
             invalid_message=invalid_message,
             transformer=transformer,
             filter=filter,
+            session_result=session_result,
+            default=default,
         )
-        self._default = default
         if not isinstance(self._default, str):
             raise InvalidArgument(
                 "default for input type question should be type of str."
@@ -80,6 +87,11 @@ class InputPrompt(BaseSimplePrompt):
         elif isinstance(completer, Completer):
             self._completer = completer
         self._multiline = multiline
+        self._complete_style = (
+            CompleteStyle.COLUMN
+            if not multicolumn_complete
+            else CompleteStyle.MULTI_COLUMN
+        )
 
         @Condition
         def is_multiline():
@@ -134,6 +146,7 @@ class InputPrompt(BaseSimplePrompt):
             lexer=SimpleLexer(self._lexer),
             is_password=kwargs.pop("is_password", False),
             multiline=self._multiline,
+            complete_style=self._complete_style,
         )
 
     def _get_prompt_message(
@@ -177,13 +190,22 @@ class InputPrompt(BaseSimplePrompt):
             )
         return formatted_message
 
-    def execute(self) -> str:
-        """Display the filepath prompt and returns the result.
+    def execute(self, raise_keyboard_interrupt: bool = True) -> Optional[str]:
+        """Display the prompt and return the result.
 
-        :return: user entered filepath
+        :param raise_keyboard_interrupt: raise kbi exception when user hit 'c-c'
+        :type raise_keyboard_interrupt: bool
+        :return: user entered value
         :rtype: str
         """
         result = self._session.prompt(default=self._default)
+        if result == INQUIRERPY_KEYBOARD_INTERRUPT:
+            if raise_keyboard_interrupt and not os.getenv(
+                "INQUIRERPY_NO_RAISE_KBI", False
+            ):
+                raise KeyboardInterrupt
+            else:
+                result = None
         if not self._filter:
             return result
         return self._filter(result)
